@@ -1,18 +1,16 @@
-# =================================================================
-#          #     #                 #     #
-#          ##    #   ####   #####  ##    #  ######   #####
-#          # #   #  #    #  #    # # #   #  #          #
-#          #  #  #  #    #  #    # #  #  #  #####      #
-#          #   # #  #    #  #####  #   # #  #          #
-#          #    ##  #    #  #   #  #    ##  #          #
-#          #     #   ####   #    # #     #  ######     #
+# ==========================================================================
+#     _   _ _ ____            ____          _____
+#    | | | (_)  _ \ ___ _ __ / ___|___  _ _|_   _| __ __ _  ___ ___ _ __
+#    | |_| | | |_) / _ \ '__| |   / _ \| '_ \| || '__/ _` |/ __/ _ \ '__|
+#    |  _  | |  __/  __/ |  | |__| (_) | | | | || | | (_| | (_|  __/ |
+#    |_| |_|_|_|   \___|_|   \____\___/|_| |_|_||_|  \__,_|\___\___|_|
 #
-#       ---   The NorNet Testbed for Multi-Homed Systems  ---
-#                       https://www.nntb.no
-# =================================================================
+#       ---  High-Performance Connectivity Tracer (HiPerConTracer)  ---
+#                 https://www.nntb.no/~dreibh/hipercontracer/
+# ==========================================================================
 #
 # High-Performance Connectivity Tracer (HiPerConTracer)
-# Copyright (C) 2015-2022 by Thomas Dreibholz
+# Copyright (C) 2015-2025 by Thomas Dreibholz
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -35,19 +33,19 @@ library("bitops")
 library("openssl")
 
 
-# ###### Convert Unix time in microseconds to string ########################
+# ###### Convert Unix time in nanoseconds to string #########################
 unix_time_to_string <- function(timeStamp)
 {
-   seconds      <- as.character(anytime(timeStamp / 1000000), asUTC=TRUE, tz="UTC")
-   microseconds <- sprintf("%06d", timeStamp %% 1000000)
-   timeString <- paste(sep=".", seconds, microseconds)
+   seconds      <- as.character(anytime(timeStamp / 1000000000), asUTC=TRUE, tz="UTC")
+   nanoseconds <- sprintf("%06d", timeStamp %% 1000000000)
+   timeString <- paste(sep=".", seconds, nanoseconds)
 }
 
 
-# ###### Convert strong to Unix time in microseconds ########################
+# ###### Convert strong to Unix time in nanoseconds #########################
 string_to_unix_time <- function(timeString)
 {
-   timeStamp <- 1000000.0 * as.numeric(anytime(timeString, asUTC=TRUE))
+   timeStamp <- 1000000000.0 * as.numeric(anytime(timeString, asUTC=TRUE))
    return(timeStamp)
 }
 
