@@ -1,18 +1,16 @@
-// =================================================================
-//          #     #                 #     #
-//          ##    #   ####   #####  ##    #  ######   #####
-//          # #   #  #    #  #    # # #   #  #          #
-//          #  #  #  #    #  #    # #  #  #  #####      #
-//          #   # #  #    #  #####  #   # #  #          #
-//          #    ##  #    #  #   #  #    ##  #          #
-//          #     #   ####   #    # #     #  ######     #
+// ==========================================================================
+//     _   _ _ ____            ____          _____
+//    | | | (_)  _ \ ___ _ __ / ___|___  _ _|_   _| __ __ _  ___ ___ _ __
+//    | |_| | | |_) / _ \ '__| |   / _ \| '_ \| || '__/ _` |/ __/ _ \ '__|
+//    |  _  | |  __/  __/ |  | |__| (_) | | | | || | | (_| | (_|  __/ |
+//    |_| |_|_|_|   \___|_|   \____\___/|_| |_|_||_|  \__,_|\___\___|_|
 //
-//       ---   The NorNet Testbed for Multi-Homed Systems  ---
-//                       https://www.nntb.no
-// =================================================================
+//       ---  High-Performance Connectivity Tracer (HiPerConTracer)  ---
+//                 https://www.nntb.no/~dreibh/hipercontracer/
+// ==========================================================================
 //
 // High-Performance Connectivity Tracer (HiPerConTracer)
-// Copyright (C) 2015-2022 by Thomas Dreibholz
+// Copyright (C) 2015-2025 by Thomas Dreibholz
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -34,16 +32,24 @@
 
 #include <stdint.h>
 
+#include <cassert>
+#include <chrono>
 #include <vector>
 #include <istream>
 
+#include "internet16.h"
+#include "resultentry.h"
+
+
+extern const ResultTimePoint HiPerConTracerEpoch;
+
 
 // ==========================================================================
 // Format:
 // 00 4 MagicNumber
 // 04 1 SendTTL
 // 05 1 Round
-// 06 2 Checksum Tweak
+// 06 2 Checksum Tweak (ICMP only) / Sequence Number (other protocols)
 // 08 8 Send Time Stamp
 // ==========================================================================
 
@@ -56,8 +62,12 @@ class TraceServiceHeader
    TraceServiceHeader(const size_t size = MIN_TRACESERVICE_HEADER_SIZE)
       : Size(size)
    {
-      assert((Size >= MIN_TRACESERVICE_HEADER_SIZE) && (Size <= MAX_TRACESERVICE_HEADER_SIZE));
-      std::fill(Data, Data + Size, 0);
+      assert((Size >= MIN_TRACESERVICE_HEADER_SIZE) &&
+             (Size <= MAX_TRACESERVICE_HEADER_SIZE));
+      std::fill(Data, Data + std::min(Size, (size_t)64), 0);
+      for(unsigned int i = 64;i < Size; i++) {
+         Data[i] = i & 0xff;
+      }
    }
 
    inline uint32_t magicNumber() const {
@@ -80,7 +90,7 @@ class TraceServiceHeader
    inline void round(uint8_t round)     { Data[5] = round;   }
 
    inline uint16_t checksumTweak() const {
-      return ( ((uint16_t)Data[6] << 8)  |
+      return ( ((uint16_t)Data[6] << 8) |
                (uint16_t)Data[7] );
    }
    inline void checksumTweak(const uint16_t value) {
@@ -88,6 +98,13 @@ class TraceServiceHeader
       Data[7] = static_cast<uint8_t>( value & 0xff );
    }
 
+   inline uint16_t seqNumber() const {
+      return checksumTweak();
+   }
+   inline void seqNumber(const uint16_t value) {
+      checksumTweak(value);
+   }
+
    inline uint64_t sendTimeStamp() const {
       return ( ((uint64_t)Data[8] << 56)  |
                ((uint64_t)Data[9] << 48)  |
@@ -108,21 +125,35 @@ class TraceServiceHeader
       Data[14] = static_cast<uint8_t>( (timeStamp >> 8) & 0xff );
       Data[15] = static_cast<uint8_t>( timeStamp & 0xff );
    }
+   inline void sendTimeStamp(const ResultTimePoint& timeStamp) {
+      // For HiPerConTracer packets: time stamp is nanoseconds since 1976-09-29.
+      // NOTE: HiPerConTracer 1.x used microseconds here. Simple heuristic:
+      // < 2*32*1e6: HiPerConTracer 1.x in microseconds
+      // Otherwise:  HiPerConTracer 2.x in nanoseconds
+      sendTimeStamp(std::chrono::duration_cast<std::chrono::nanoseconds>(
+                       timeStamp - HiPerConTracerEpoch).count());
+   }
+
+   inline void computeInternet16(uint32_t& sum) const {
+      ::computeInternet16(sum, (uint8_t*)&Data, Size);
+   }
+
+   inline const uint8_t* data() const {
+      return (const uint8_t*)&Data;
+   }
+   inline size_t size() const {
+      return Size;
+   }
 
    inline friend std::istream& operator>>(std::istream& is, TraceServiceHeader& header) {
-      return(is.read(reinterpret_cast<char*>(header.Data), header.Size));
+      return is.read(reinterpret_cast<char*>(header.Data), header.Size);
    }
 
    inline friend std::ostream& operator<<(std::ostream& os, const TraceServiceHeader& header) {
-      return(os.write(reinterpret_cast<const char*>(header.Data), header.Size));
+      return os.write(reinterpret_cast<const char*>(header.Data), header.Size);
    }
 
-   inline std::vector<uint8_t> contents() const {
-      std::vector<uint8_t> contents((uint8_t*)&Data, (uint8_t*)&Data[Size]);
-      return(contents);
-   }
-
-//    private:
+   private:
    const size_t Size;
    uint8_t      Data[MAX_TRACESERVICE_HEADER_SIZE];
 };
